import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt
from math import*


def main():
    # Poynomial interpolation in Lagrange form.
    #
    # Input:   x : vector with the nodes
    #          y : Vector with the corresponding y-values.
    #      [a,b] : Interpolation interval
    # Output:  t : vector of equally spaced points in [a,b]
    #          p : the value of the interpolation
    #              polynomial in [a,b].

    x = [0,0.4,0.6,0.9]
    y = [1,sqrt(1.4),sqrt(1.6),sqrt(1.9)]
    a = -1
    b = 2
    t,p = Lagrange(x,y,a,b)

    fig, ax = plt.subplots()
    ax.plot(t,p,color="blue")
    ax.plot(t,np.sqrt(t+np.ones(len(t))),color="red")
    fig.savefig("1c.png", dpi=100, facecolor="#f1f1f1")


def Lagrange(x,y,a,b):
    n = len(x)
    t = np.linspace(a,b,101)
    nt = len(t)
    p = np.zeros(nt)
    L = Cardinal(x,t)
    for k in range(0,n):
        for i in range(0,nt):
            p[i] += y[k]*L[i][k]
    return t,p


def Cardinal(x,t):
    # The cardinal functions lk of x, evaluated in t.
    # t is a column vector.
    # Output: L = [l1(t),l2(t),....,ln(t)]

    n = len(x)
    nt = len(t)
    L = np.ones((nt,n))
    for k in range(0,n):
        for i in range(0,n):
            if (i != k):
                for j in range(0,nt):
                    d = (t[j]-x[i])/(x[k]-x[i])
                    L[j][k] *= d
    return L


main()
